"""
Document ingestion module for Verbatim RAG.

Simple integration of docling + chonkie for document processing.
"""

from .document_processor import DocumentProcessor

__all__ = ["DocumentProcessor"]
